﻿<#
    .SYNOPSIS
    Modifies SCSM permissions to allow Activity Implementers to edit the ActualEndDate property

    .DESCRIPTION
    SCSM 2012 R2 UR7 introduced a fix due to which SCSM will properly set the "ActualEndDate" property on work items at time of resolution.
    
    Unfortunately, Activity Implementers' permissions were not modified to give them permission to set the "ActualEndDate" property, resulting in the Activity Implementers user role being broken after the update was applied - members of the user role could no longer perform the one action their user role was meant to allow, that being the Completion of Manual Activities.

    A fix for the bug will be released as part of SCSM 2016. However, until that time and for those who are unable to upgrade, this script provides a workaround.

    This script will grant the Activity Implementers user role (and any user role based on it) permissions to update the "ActualEndDate" property of work items, allowing them to properly complete Manual Activities.

    A Service Manager administrator only needs to run the script on a Service Manager management server and the permissions will be added.

    .NOTES
    Written by Gabriel Taylor and Mihai Sarbulescu, 2016
#>

#region Parameters
[CmdletBinding()]
param ()
#endregion

#region Variables
[datetime]$StartTime = Get-Date
[string]$SMServer = 'localhost'
[string]$RoleName = 'ActivityImplementer'
[string]$ClassName = 'System.WorkItem'
[string]$PropertyName = 'ActualEndDate'
[string]$SMModule = 'System.Center.Service.Manager'
[string]$SMNamespace = 'Microsoft.EnterpriseManagement.EnterpriseManagementGroup'
[string]$RelEndPoint = 'Microsoft.EnterpriseManagement.Security.RelationshipEndpoint'
[string]$OperationImplicitScope = 'Microsoft.EnterpriseManagement.Security.OperationImplicitScope'
#endregion

#region Start up
Write-Verbose "ImplementUR7ActivityImplementerWorkaround.ps1" -Verbose
Write-Verbose "This script will extend the permissions of Activity Implementers to enable them to complete manual activities." -Verbose
Write-Verbose "This is a workaround for a bug introduced in SCSM 2012 R2 UR7." -Verbose
Write-Verbose "This script MUST be run on an SCSM management server." -Verbose
Write-Verbose "This script MUST be run by an SCSM administrator." -Verbose
Write-Verbose "If errors occur, validate that the above requirements are true and that no other custom permissions have been configured, then try again." -Verbose
Write-Verbose "Beginning permissions configuration ..." -Verbose
#endregion

#region Retrieve the SCSM install directory
Write-Verbose -Message "Retrieving SCSM install path from the registry ..."
$SCSMRegistryKey = Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\System Center\2010\Common\Setup"
$InstallDirectory = $SCSMRegistryKey.InstallDirectory.TrimEnd('\')
#endregion

#region Connect to the SCSM Management Group
## Add the SCSM DLL in order to interact with the SDK
Write-Verbose -Message "Loading the SCSM SDK ..."
$SDKDLLPath = $InstallDirectory + '\SDK Binaries\Microsoft.EnterpriseManagement.Core.dll'
Add-Type -Path $SDKDLLPath

## Connect to the Management Group
Write-Verbose -Message "Connecting to the management group on $SMServer ..."
$EMGType = $SMNamespace + '.EnterpriseManagementGroup'
$EMG = New-Object -TypeName $EMGType -ArgumentList $SMServer
#endregion

#region Identify the Profile and Class
## Get the user profile that should be changed
Write-Verbose -Message "Retrieving information for user role profile $RoleName ..."
$Profile = $EMG.Security.GetProfiles() | ?{$_.Name -eq $RoleName}

## Get class
Write-Verbose -Message "Retrieving information for class $ClassName ..."
$Class = $EMG.EntityTypes.GetClasses() | ?{$_.Name -eq $ClassName}
#endregion

#region Add permissions to update a class property
## Retrieve the property information
Write-Verbose -Message "Retrieving information for property $PropertyName ..."
$Property = $Class.GetProperties() | ?{$_.Name -eq $PropertyName}

## Get the Object__Set implicit scope
Write-Verbose -Message "Configuring custom permissions ..."
$ObjSet = $Profile.Operations | ?{$_.Name -eq "Object__Set"}

## Create the new implicit scope object for the class + property combination
$OIObject = New-Object $OperationImplicitScope -ArgumentList @($Class.Id,$Property.Id)

## Add the new scope object to the array of scopes, then update the profile
Write-Verbose -Message "Setting custom permissions ..."
$ObjSet.ImplicitScopes.Add($OIObject)
$Profile.Update()
#endregion 

#region Wrap-up
Write-Verbose -Message "Permissions configuration complete!" -Verbose
Write-Verbose -Message "Total time: $(New-TimeSpan -Start $StartTime -End Get-Date)" -Verbose
#endregion